# -*- coding: utf-8 -*-

"""
======================================================================================
ftp server

if pyftpdlib is not installed, install it first! (pip install pyftpdlib)

reference: http://www.cnblogs.com/huangxm/p/6274645.html


读权限 ：
e 	改变文件目录
l 	列出文件
r 	从服务器接收文件

写权限 ：
a 	文件上传
d 	删除文件
f 	文件重命名
m 	创建文件
w 	写权限
M 	文件传输模式（通过FTP设置文件权限 ）
======================================================================================
"""

from pyftpdlib.authorizers import DummyAuthorizer
from pyftpdlib.handlers import FTPHandler, ThrottledDTPHandler
from pyftpdlib.servers import FTPServer
import logging

"""
======================================================================================
setting
======================================================================================
"""


class settings:
    ip = None
    if not ip:
        ip

    port = '21'

    user_list = [
        # 用户名   密码       权限       目录
        ['root', '12345', 'elradfmwM', 'C:\ZB'],
    ]

    # 上传速度  300kb/s
    max_upload = 3000 * 1024

    # 下载速度 300kb/s
    max_download = 3000 * 1024

    # 最大连接数
    max_cons = 150

    # 最多IP数
    max_per_ip = 10

    # 被动端口范围，注意被动端口数量要比最大IP数多，否则可能出现无法连接的情况
    passive_ports = (2000, 2200)

    # 是否开启匿名访问 on|off
    enable_anonymous = 'off'

    # 匿名用户目录
    anonymous_path = ''

    # 是否开启日志 on|off
    enable_logging = 'on'
    # 日志文件
    logging_name = './pyftp.log'

    # 欢迎信息
    welcome_msg = 'Welcome to my ftp'


def ftp_server():
    # 实例化虚拟用户，这是FTP验证首要条件
    authorizer = DummyAuthorizer()

    for user in settings.user_list:
        name, passwd, permit, homedir = user
        try:
            authorizer.add_user(name, passwd, homedir, perm=permit)
        except Exception as e:
            print(e)

    # 添加匿名用户 只需要路径
    if settings.enable_anonymous == 'on':
        authorizer.add_anonymous(settings.anonymous_path)

    # 下载上传速度设置
    dtp_handler = ThrottledDTPHandler
    dtp_handler.read_limit = settings.max_download
    dtp_handler.write_limit = settings.max_upload

    # 初始化ftp句柄
    handler = FTPHandler
    handler.authorizer = authorizer

    # 日志记录
    if settings.enable_logging == 'on':
        logging.basicConfig(filename=settings.logging_name, level=logging.INFO)

    # 欢迎信息
    handler.banner = settings.welcome_msg

    # 添加被动端口范围
    handler.passive_ports = range(settings.passive_ports[0], settings.passive_ports[1])

    # 监听ip 和 端口
    server = FTPServer((settings.ip, settings.port), handler)

    # 最大连接数
    server.max_cons = settings.max_cons
    server.max_cons_per_ip = settings.max_per_ip

    # 开始服务
    server.serve_forever()


if __name__ == "__main__":
    ftp_server()



# -*- coding: utf-8 -*-

import sys

class ProgressBar:
    """
    import time
    bar = ProgressBar(total = 10)
    for i in range(10):
        bar.move()
        bar.log()
        time.sleep(1)
    """

    def __init__(self, count=0, total=0, width=50):
        self.count = count
        self.total = total
        self.width = width

    def move(self):
        self.count += 1

    def log(self):
        sys.stdout.write('' * (self.width + 9) + '\r')
        sys.stdout.flush()
        progress = self.width * self.count / self.total
        sys.stdout.write('{0}/{1}: '.format(self.count, self.total))
        sys.stdout.write('#' * int(progress) + '-' * int(self.width - progress) + '\r')
        if progress == self.width:
            sys.stdout.write('\n')
        sys.stdout.flush()



# -*- coding: utf-8 -*-

import click


@click.group()
def zb():
    pass

@zb.command()
@click.option('-p', '--path', help="文件路径")
@click.option('-f', '--f', help="文件名中需要修改的内容")
@click.option('-t', '--t', help="修改之后的内容")
def brf(path, f, t):
    """batch_rename_file | 批量重命名文件"""
    from .tools.file_tools import batch_rename_file
    batch_rename_file(path, f, t)

if __name__ == "__main__":
    zb()




# -*- coding: utf-8 -*-

"""
————————————————————————————————————————————————————————————————————————————————————————
豆瓣网爬虫

————————————————————————————————————————————————————————————————————————————————————————
"""

import os
import requests
import urllib
from bs4 import BeautifulSoup
from PIL import Image

from zb_CodeSet.Spiders.utils import get_header

"""
————————————————————————————————————————————————————————————————————————————————————————
模拟登陆
1. use post
2. use cookies
————————————————————————————————————————————————————————————————————————————————————————
"""


def login_with_post(user, pw):
    """模拟登录：使用post方法传递用户名和密码"""
    session = requests.Session()
    session.headers.update(get_header())

    # 构造登录需要的信息
    data = {
        'source': 'index_nav',
        'redir': 'http://www.douban.com',
        'form_email': user,
        'form_password': pw,
        'login': '登录',
    }

    # 获取验证码
    url = 'https://accounts.douban.com/login'
    html = requests.get(url)
    html = BeautifulSoup(html.text, 'lxml')
    caprcha_link = html.select('#captcha_image')[0]['src']
    caprcha_id = html.select('div.captcha_block > input')[1]['value']
    fn_jpg = 'caprcha.jpg'
    if caprcha_id:  # 如果有caprcha_id,就执行解析caprcha_link网页信息，并把图片保存下来打开
        img_html = session.get(caprcha_link)
        with open(fn_jpg, 'wb') as f:
            f.write(img_html.content)
        try:
            im = Image.open(fn_jpg)
            im.show()
            im.close()
            os.remove(fn_jpg)
        except:
            print('打开错误')
        caprcha = input('请输入验证码：')  # 把看到的验证码图片输入进去

    if caprcha_id:  # 如果需要验证码就把下面的两个数据加入到data里面
        data['captcha-id'] = caprcha_id
        data['captcha-solution'] = caprcha

    # 登录
    session.post(url, data=data)
    if session.cookies:
        print(' %s 登录成功！\n cookies： ' % user, session.cookies.items())
    else:
        print('%s 登录失败，请重试！' % user)

    return session


def login_with_cookies(raw_cookies):
    """创建cookies

    example
    -----------
    raw_cookies = 'bid=tiyTMxpIffk; ll="118282"; ps=y; ue="1257391203@qq.com"; dbcl2="143012014:C8XrOzYKwJc"; push_noty_num=0; push_doumail_num=0; __ads_session=cfktlbLf+Ags5WswgQA=; ck=FBAI; ap=1'
    session = login_with_cookies(raw_cookies)
    """
    cookies = {}
    for line in raw_cookies.split(';'):
        key, value = line.split('=', 1)
        cookies[key.strip(' ')] = value

    session = requests.Session()
    session.cookies.update(cookies)
    session.headers.update(get_header())

    return session


"""
————————————————————————————————————————————————————————————————————————————————————————
豆瓣小组
————————————————————————————————————————————————————————————————————————————————————————
"""


def parse_douban_group(session, page1):
    # page1 = 'https://www.douban.com/group/'
    group = session.get(page1)
    soup = BeautifulSoup(group.text, 'lxml')
    titles = soup.select('tr.pl > td.td-subject > a.title')
    for title in titles:
        print(title['href'], title.string)
    return titles


"""
————————————————————————————————————————————————————————————————————————————————————————
豆瓣阅读
https://read.douban.com/search?q=Python&start=
————————————————————————————————————————————————————————————————————————————————————————
"""


def parse_douban_read():
    # 拼接urls（所有url）
    urls = []
    for i in (0, 10, 20):
        urls.append('https://read.douban.com/search?q=PYTHON&start=' + str(i))
    print(urls)

    for url in urls:
        # 读取网页内容
        req = urllib.request.Request(url)
        res = urllib.request.urlopen(req).read().decode('utf-8')
        # 解析网页
        soup = BeautifulSoup(res, 'lxml')
        # 循环的方式读取所需字段
        for content in soup.select('.info'):
            title = content.select('.title')[0].text
            if not content.select('.author-item'):
                anthor = None
            else:
                author = content.select('.author-item')[0].text
            price = content.select('.price-tag')[0].text
            category = content.select('.category')[0].text[3:]
            if not content.select('.rating-average'):
                rating = None
            else:
                rating = content.select('.rating-average')[0].text
            if not content.select('.ratings-link'):
                eveluate_nums = None
            else:
                eveluate_nums = content.select('.ratings-link')[0].text[:-3]
            if not content.select('.article-desc-brief'):
                desc = None
            else:
                desc = content.select('.article-desc-brief')[0].text
            print(title, author, price, category, rating, eveluate_nums, desc)



# -*- coding: utf-8 -*-
"""
python module fast migration
"""
import os

# step 1. get module installed list
def get_module_list(upgrade=False):
    requirements = r'c:\module_installed.txt'
    os.system('pip list >> ' + requirements)

    # read file, modify format
    with open(requirements, 'r') as f:
        lines = f.readlines()
        if not upgrade:
            modules = [i.replace(')', '').replace('(', '== ') for i in lines]
        else:
            modules = [i.replace(')', '').replace('(', '>= ') for i in lines]

    # write results to file
    with open(requirements, 'w') as f:
        f.writelines(modules)

# step 2. install modules in another machine
def install_module_list():
    requirements = r'c:\module_installed.txt'
    os.system('pip install -r ' + requirements)

def install_module_list_without_stop():
    requirements = r'c:\module_installed.txt'
    with open(requirements, 'r') as f:
        module_list = f.readlines()
    for lib in module_list:
        try:
            os.system('pip install ' + lib.strip('\n'))
        except Exception:
            print('pip 安装模块 %s 出错' % lib.strip('\n'))



# -*- coding: utf-8 -*-
"""
——————————————————————————————————————————————————————————————————————————————————————————
url下载器模块

返回html源码
——————————————————————————————————————————————————————————————————————————————————————————
"""


from urllib.request import urlopen
import requests
import random


"""
——————————————————————————————————————————————————————————————————————————————————————————
USER_AGENTS 随机头信息
——————————————————————————————————————————————————————————————————————————————————————————
"""
USER_AGENTS = [
    "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; AcooBrowser; .NET CLR 1.1.4322; .NET CLR 2.0.50727)",
    "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Acoo Browser; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.0.04506)",
    "Mozilla/4.0 (compatible; MSIE 7.0; AOL 9.5; AOLBuild 4337.35; Windows NT 5.1; .NET CLR 1.1.4322; .NET CLR 2.0.50727)",
    "Mozilla/5.0 (Windows; U; MSIE 9.0; Windows NT 9.0; en-US)",
    "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Win64; x64; Trident/5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Media Center PC 6.0)",
    "Mozilla/5.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 1.0.3705; .NET CLR 1.1.4322)",
    "Mozilla/4.0 (compatible; MSIE 7.0b; Windows NT 5.2; .NET CLR 1.1.4322; .NET CLR 2.0.50727; InfoPath.2; .NET CLR 3.0.04506.30)",
    "Mozilla/5.0 (Windows; U; Windows NT 5.1; zh-CN) AppleWebKit/523.15 (KHTML, like Gecko, Safari/419.3) Arora/0.3 (Change: 287 c9dfb30)",
    "Mozilla/5.0 (X11; U; Linux; en-US) AppleWebKit/527+ (KHTML, like Gecko, Safari/419.3) Arora/0.6",
    "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.8.1.2pre) Gecko/20070215 K-Ninja/2.1.1",
    "Mozilla/5.0 (Windows; U; Windows NT 5.1; zh-CN; rv:1.9) Gecko/20080705 Firefox/3.0 Kapiko/3.0",
    "Mozilla/5.0 (X11; Linux i686; U;) Gecko/20070322 Kazehakase/0.4.5",
    "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.0.8) Gecko Fedora/1.9.0.8-1.fc10 Kazehakase/0.5.6",
    "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.56 Safari/535.11",
    "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_3) AppleWebKit/535.20 (KHTML, like Gecko) Chrome/19.0.1036.7 Safari/535.20",
    "Opera/9.80 (Macintosh; Intel Mac OS X 10.6.8; U; fr) Presto/2.9.168 Version/11.52",
    "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.11 TaoBrowser/2.0 Safari/536.11",
    "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.71 Safari/537.1 LBBROWSER",
    "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; LBBROWSER)",
    "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; QQDownload 732; .NET4.0C; .NET4.0E; LBBROWSER)",
    "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.84 Safari/535.11 LBBROWSER",
    "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)",
    "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; QQBrowser/7.0.3698.400)",
    "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; QQDownload 732; .NET4.0C; .NET4.0E)",
    "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; SV1; QQDownload 732; .NET4.0C; .NET4.0E; 360SE)",
    "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; QQDownload 732; .NET4.0C; .NET4.0E)",
    "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)",
    "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1",
    "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1",
    "Mozilla/5.0 (iPad; U; CPU OS 4_2_1 like Mac OS X; zh-cn) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C148 Safari/6533.18.5",
    "Mozilla/5.0 (Windows NT 6.1; Win64; x64; rv:2.0b13pre) Gecko/20110307 Firefox/4.0b13pre",
    "Mozilla/5.0 (X11; Ubuntu; Linux x86_64; rv:16.0) Gecko/20100101 Firefox/16.0",
    "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.64 Safari/537.11",
    "Mozilla/5.0 (X11; U; Linux x86_64; zh-CN; rv:1.9.2.10) Gecko/20100922 Ubuntu/10.10 (maverick) Firefox/3.6.10"
]

def get_header():
    return {
        'User-Agent': random.choice(USER_AGENTS),
        'Accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8',
        'Accept-Language': 'en-US,en;q=0.5',
        'Connection': 'keep-alive',
        'Accept-Encoding': 'gzip, deflate',
    }


"""
——————————————————————————————————————————————————————————————————————————————————————————
使用urlopen下载
——————————————————————————————————————————————————————————————————————————————————————————
"""


def download_with_urlopen(url):
    return urlopen(url)


"""
——————————————————————————————————————————————————————————————————————————————————————————
使用requests下载
——————————————————————————————————————————————————————————————————————————————————————————
"""


def download_with_requests(url, method='get',
                           proxies=None, timeout=3):
    """使用requests下载"""
    # 获取一个随机header
    header = get_header()
    if method == 'get':
        html = requests.get(url, headers=header, proxies=proxies, timeout=timeout)
    elif method == 'post':
        html = requests.post(url, headers=header, proxies=proxies, timeout=timeout)
    return html.text







# -*- coding: utf-8 -*-

from numpy import *


def loadDataSet(fileName):
    dataMat = []
    fr = open(fileName)
    for line in fr.readlines():
        curLine = line.strip().split('\t')
        fltLine = map(float, curLine)
        dataMat.append(fltLine)
    return dataMat


# 计算两个向量的欧几里得距离
def distEclud(vecA, vecB):
    return sqrt(sum(power(vecA - vecB, 2)))


# 随机生成初始的类簇中心
def randCent(dataSet, k):
    n = shape(dataSet)[1]
    centroids = mat(zeros((k, n)))
    for j in range(n):
        minJ = min(dataSet[:, j])
        rangeJ = float(max(array(dataSet)[:, j]) - minJ)
        centroids[:, j] = minJ + rangeJ * random.rand(k, 1)
    return centroids


def kMeans(dataSet, k, distMeas=distEclud, createCent=randCent):
    m = shape(dataSet)[0]
    clusterAssment = mat(zeros((m, 2)))  # create mat to assign data points
    # to a centroid, also holds SE of each point
    centroids = createCent(dataSet, k)
    clusterChanged = True
    while clusterChanged:
        clusterChanged = False
        for i in range(m):  # for each data point assign it to the closest centroid
            minDist = inf
            minIndex = -1
            for j in range(k):
                distJI = distMeas(centroids[j, :], dataSet[i, :])
                if distJI < minDist:
                    minDist = distJI
                    minIndex = j
            if clusterAssment[i, 0] != minIndex:
                clusterChanged = True
            clusterAssment[i, :] = minIndex, minDist ** 2
        print(centroids)
        for cent in range(k):  # recalculate centroids
            ptsInClust = dataSet[nonzero(clusterAssment[:, 0].A == cent)[0]]  # get all the point in this cluster
            centroids[cent, :] = mean(ptsInClust, axis=0)  # assign centroid to mean
    return centroids, clusterAssment


def show(dataSet, k, centroids, clusterAssment):
    from matplotlib import pyplot as plt
    numSamples, dim = dataSet.shape
    mark = ['or', 'ob', 'og', 'ok', '^r', '+r', 'sr', 'dr', '<r', 'pr']
    for i in range(numSamples):
        markIndex = int(clusterAssment[i, 0])
        plt.plot(dataSet[i, 0], dataSet[i, 1], mark[markIndex])
    mark = ['Dr', 'Db', 'Dg', 'Dk', '^b', '+b', 'sb', 'db', '<b', 'pb']
    for i in range(k):
        plt.plot(centroids[i, 0], centroids[i, 1], mark[i], markersize=12)
    plt.show()


def main():
    dataMat = mat(loadDataSet('testSet.txt'))
    myCentroids, clustAssing = kMeans(dataMat, 4)
    print(myCentroids)
    show(dataMat, 4, myCentroids, clustAssing)


if __name__ == '__main__':
    main()




# -*- coding: utf-8 -*-

import os
import json
import requests
from bs4 import BeautifulSoup
import random


USER_AGENTS = [
    "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; AcooBrowser; .NET CLR 1.1.4322; .NET CLR 2.0.50727)",
    "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Acoo Browser; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.0.04506)",
    "Mozilla/4.0 (compatible; MSIE 7.0; AOL 9.5; AOLBuild 4337.35; Windows NT 5.1; .NET CLR 1.1.4322; .NET CLR 2.0.50727)",
    "Mozilla/5.0 (Windows; U; MSIE 9.0; Windows NT 9.0; en-US)",
    "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Win64; x64; Trident/5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Media Center PC 6.0)",
    "Mozilla/5.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 1.0.3705; .NET CLR 1.1.4322)",
    "Mozilla/4.0 (compatible; MSIE 7.0b; Windows NT 5.2; .NET CLR 1.1.4322; .NET CLR 2.0.50727; InfoPath.2; .NET CLR 3.0.04506.30)",
    "Mozilla/5.0 (Windows; U; Windows NT 5.1; zh-CN) AppleWebKit/523.15 (KHTML, like Gecko, Safari/419.3) Arora/0.3 (Change: 287 c9dfb30)",
    "Mozilla/5.0 (X11; U; Linux; en-US) AppleWebKit/527+ (KHTML, like Gecko, Safari/419.3) Arora/0.6",
    "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.8.1.2pre) Gecko/20070215 K-Ninja/2.1.1",
    "Mozilla/5.0 (Windows; U; Windows NT 5.1; zh-CN; rv:1.9) Gecko/20080705 Firefox/3.0 Kapiko/3.0",
    "Mozilla/5.0 (X11; Linux i686; U;) Gecko/20070322 Kazehakase/0.4.5",
    "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.0.8) Gecko Fedora/1.9.0.8-1.fc10 Kazehakase/0.5.6",
    "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.56 Safari/535.11",
    "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_3) AppleWebKit/535.20 (KHTML, like Gecko) Chrome/19.0.1036.7 Safari/535.20",
    "Opera/9.80 (Macintosh; Intel Mac OS X 10.6.8; U; fr) Presto/2.9.168 Version/11.52",
    "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.11 TaoBrowser/2.0 Safari/536.11",
    "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.71 Safari/537.1 LBBROWSER",
    "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; LBBROWSER)",
    "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; QQDownload 732; .NET4.0C; .NET4.0E; LBBROWSER)",
    "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.84 Safari/535.11 LBBROWSER",
    "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)",
    "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; QQBrowser/7.0.3698.400)",
    "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; QQDownload 732; .NET4.0C; .NET4.0E)",
    "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; SV1; QQDownload 732; .NET4.0C; .NET4.0E; 360SE)",
    "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; QQDownload 732; .NET4.0C; .NET4.0E)",
    "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)",
    "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1",
    "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1",
    "Mozilla/5.0 (iPad; U; CPU OS 4_2_1 like Mac OS X; zh-cn) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C148 Safari/6533.18.5",
    "Mozilla/5.0 (Windows NT 6.1; Win64; x64; rv:2.0b13pre) Gecko/20110307 Firefox/4.0b13pre",
    "Mozilla/5.0 (X11; Ubuntu; Linux x86_64; rv:16.0) Gecko/20100101 Firefox/16.0",
    "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.64 Safari/537.11",
    "Mozilla/5.0 (X11; U; Linux x86_64; zh-CN; rv:1.9.2.10) Gecko/20100922 Ubuntu/10.10 (maverick) Firefox/3.6.10"
]


def get_header():
    return {
        'User-Agent': random.choice(USER_AGENTS),
        'Accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8',
        'Accept-Language': 'en-US,en;q=0.5',
        'Connection': 'keep-alive',
        'Accept-Encoding': 'gzip, deflate',
    }


def network_ready():
    """判断当前网络是否可用"""
    is_connect = os.system('ping wwww.baidu.com')
    if is_connect != 0:
        return False
    else:
        return True


def get_self_ip():
    """获取本机公网ip地址"""
    TEST_IP = 'http://httpbin.org/ip'
    html = requests.get(TEST_IP)
    ip = json.loads(html.text)['origin']
    return ip


def get_ip_location(ip):
    """获取ip的地理位置信息"""
    ip = 'http://ip.chinaz.com/' + ip
    html = requests.get(ip)
    ip_info = BeautifulSoup(html.text, 'lxml')
    ip_loc = ip_info.find_all('span', {'class': 'Whwtdhalf w50-0'})
    return ip_loc[-1].text


def get_self_location():
    """获取本机网络当前地理位置"""
    ip = get_self_ip()
    location = get_ip_location(ip)
    return location




# -*- coding: utf-8 -*-

import os

try:
    import docx
except ImportError:
    os.system('pip install python-docx')

# 新建文档
doc_new = docx.Document()

# 读取文档
f = r'f:\Data_Warehouse\ProbeInfo\BaoAn_Sample\WangGe\共享库数据字典v1.3.docx'
doc = docx.Document(f)





# -*- coding: utf-8 -*-

"""
preprocessing data for time series
===================================================================================





===================================================================================
"""

import pandas as pd
import numpy
from sklearn.preprocessing import MinMaxScaler

def timeseries_to_supervised(df, lag=1):
    columns = [df.shift(i) for i in range(1, lag+1)]
    columns.append(df)
    df = pd.concat(columns, axis=1)
    df.fillna(0, inplace=True)
    return df


def difference(dataset, interval=1):
    diff = list()
    for i in range(interval, len(dataset)):
        value = dataset[i] - dataset[i - interval]
        diff.append(value)
    return pd.Series(diff)


def inverse_difference(history, yhat, interval=1):
    return yhat + history[-interval]


# scale train and test data to [-1, 1]
def scale(train, test):
    # fit scaler
    scaler = MinMaxScaler(feature_range=(-1, 1))
    scaler = scaler.fit(train)
    # transform train
    train = train.reshape(train.shape[0], train.shape[1])
    train_scaled = scaler.transform(train)
    # transform test
    test = test.reshape(test.shape[0], test.shape[1])
    test_scaled = scaler.transform(test)
    return scaler, train_scaled, test_scaled


# inverse scaling for a forecasted value
def invert_scale(scaler, X, value):
    new_row = [x for x in X] + [value]
    array = numpy.array(new_row)
    array = array.reshape(1, len(array))
    inverted = scaler.inverse_transform(array)
    return inverted[0, -1]





# -*- coding: utf-8 -*-
"""
————————————————————————————————————————————————————————————————————————————————————————
花瓣网爬虫 -- 批量获取美图

页面分析：
1、home  http://huaban.com/
2、http://huaban.com/discovery/
3、http://huaban.com/pins/68106074/

————————————————————————————————————————————————————————————————————————————————————————
"""



# import requests
from selenium import webdriver
from urllib.request import quote
from urllib.request import urlopen
from zb_CodeSet.Spiders.utils import get_header
from bs4 import BeautifulSoup
import requests


img = 'http:' + '//img.hb.aicdn.com/5a94edc3a30607dcc881af5d7a9c2d696b16203d29407-8fOTcH_fw658'

def save_pic(img_url, name):
    """保存图片"""
    pic = urlopen(img_url)
    with open(name, 'wb') as f:
        f.writelines(pic.readlines())


def search_url(key='笑', sort=None):
    """花瓣搜索，下载图片

    params
    ---------
    key     str
            搜索关键词，如：‘笑’
    sort    weight, relative, created_at  默认 None
            搜索结果的排序方法
            None = 综合
            weight = 热门
            relative = 匹配度
            created_at = 时间
    """
    if sort is None:
        url = 'http://huaban.com/search/?q=' + quote(key)
    else:
        url = 'http://huaban.com/search/?q=' + quote(key) + '&sort=' + sort

    return url


def download_url(url):
    driver = webdriver.PhantomJS()
    driver.get(url)
    bsObj = BeautifulSoup(driver.page_source, 'lxml')
    return bsObj



# -*- coding: utf-8 -*-
"""
Download file from web.
"""

import os
import os.path
from urllib.request import urlopen
# from urllib.request import urlretrieve
try:
    import wget
except ImportError:
    status = os.system('pip install wget')
    import wget


rar = 'http://www102.zippyshare.com/d/kbL2rs9I/12271/For.Dummies.Excel.VBA.Programming.For.Dummies.4th.Edition.1119077397.rar'
pdf = 'http://www.ndrc.gov.cn/fzgggz/fzgh/ghwb/dfztgh/201606/P020160622418453985437.pdf'
mp4 = ''
mp3 = 'http://www.futurecrew.com/skaven/song_files/mp3/razorback.mp3'
pic = ''

"""
——————————————————————————————————————————————————————————————————————————————
use urlopen
——————————————————————————————————————————————————————————————————————————————
"""
def download(url):
    """输入文件url，下载该文件"""
    file_name = os.path.split(url)[1]
    file = urlopen(url)
    with open(file_name, 'wb') as f:
        f.write(file.read())
        print("文件下载成功：%s " % file_name)


"""
——————————————————————————————————————————————————————————————————————————————
use wget
——————————————————————————————————————————————————————————————————————————————
"""

filename = wget.download(pdf)




# -*- coding: utf-8 -*-
"""
IP代理自动获取，并检查可用性
"""

import requests
from zb_CodeSet.Spiders.url_downloader import download_with_requests
from bs4 import BeautifulSoup


"""
——————————————————————————————————————————————————————————————————————————————————————————
从xicidaili爬取免费代理
——————————————————————————————————————————————————————————————————————————————————————————
"""

def parser_xicidaili(xi_url):
    """解析xicidaili的url链接，获取可用的ip"""
    html = download_with_requests(xi_url)
    html = BeautifulSoup(html, 'lxml')

    ips = html.find('table', {'id': 'ip_list'})
    ips = ips.find_all('tr')
    ip_list = []
    for ip in ips[1:]:
        ip = [i.strip(' ') for i in ip.text.split('\n') if i != '']
        ip = ip[:2]
        ip_list.append(ip)
    return ip_list


def get_ips_from_xicidaili(page_num=3):
    """从xicidaili爬取免费代理(返回结果未验证可用性)"""
    url = 'http://www.xicidaili.com/nn/'
    ips = []
    for i in range(page_num):
        xi_url = url + str(i)
        try:
            ip_list = parser_xicidaili(xi_url)
            ips += ip_list
        except:
            continue
    return ips



"""
——————————————————————————————————————————————————————————————————————————————————————————
测试免费代理的可用性
——————————————————————————————————————————————————————————————————————————————————————————
"""
def is_usable(addr, port, timeout=3):
    """测试代理是否可用

    params
    ----------
    addr        ip地址
    port        端口号
    timeout     默认值为3，通过设置这个参数可以过滤掉一些速度慢的代理

    example
    ----------
    is_usable('222.180.24.13', '808', timeout=3)

    """
    try:
        proxies = {
            'http': 'http://%s:%s' % (addr, port),
            'https': 'https://%s:%s' % (addr, port)
        }
        requests.get('http://www.baidu.com/', proxies=proxies, timeout=timeout)
    except:
        print('failed: ', addr, port)
    else:
        print('success: ', addr, port)
        return addr, port


def check_ips(ips):
    """check all ip in ips, find usable ip"""
    ips_usable = []
    for ip in ips:
        # print(ip)
        try:
            addr, port = is_usable(ip[0], ip[1])
            ips_usable.append([addr, port])
        except:
            continue
    return ips_usable


"""
——————————————————————————————————————————————————————————————————————————————————————————
获取可用代理的主函数
——————————————————————————————————————————————————————————————————————————————————————————
"""
def get_IPs_in_xicidaili(page_num=3):
    """返回xicidaili中可用的ips"""
    ips = get_ips_from_xicidaili(page_num=page_num)  # len(ips)
    ips_usable = check_ips(ips)               # len(ips_usable)
    return ips_usable


if __name__ == "__main__":
    pass





# -*- coding: utf-8 -*-
"""
===================================================================================
LSTMs model

input   numpy.ndarray(1-D)

"""

import numpy as np
import tensorflow as tf
from tensorflow.contrib.learn.python.learn.estimators.estimator import SKCompat
import matplotlib.pyplot as plt
learn = tf.contrib.learn


"""
========================================================================================
data preprocessing
"""

def timeseries_to_supervised(seq, timesteps=10):
    """transform seq to supervised datas"""
    X = []
    y = []
    for i in range(len(seq) - timesteps - 1):
        X.append([seq[i: i + timesteps]])
        y.append([seq[i + timesteps]])
    return np.array(X, dtype=np.float32), np.array(y, dtype=np.float32)

# input data
import pandas as pd
f_csv = r"C:\DataWarehouse\sszt\2010_sszt_new.csv"
sszt_new = pd.read_csv(f_csv, index_col=[0], parse_dates=[0], encoding='gbk')
sszt_new = sszt_new.sort_index()

# split data & transform
new_value = sszt_new.values
X, y = timeseries_to_supervised(new_value.reshape(new_value.shape[0]), timesteps=1)
from sklearn.model_selection import train_test_split
train_X, test_X, train_y, test_y = train_test_split(X, y, test_size=0.2)



"""
========================================================================================
create model
"""

# set parameters
HIDDEN_SIZE = 30    # 隐层神经元数量
NUM_LAYERS = 6     # 网络层数
TRAINING_STEPS = 3000
BATCH_SIZE = 6
KEEP_PROB = 0.5


def lstm_model(X, y):
    """define a lstm model"""
    def lstm_cell():
        if KEEP_PROB < 1:
            return tf.contrib.rnn.DropoutWrapper(
                tf.contrib.rnn.BasicLSTMCell(HIDDEN_SIZE, state_is_tuple=True),
                output_keep_prob=KEEP_PROB)
        else:
            return tf.contrib.rnn.BasicLSTMCell(HIDDEN_SIZE, state_is_tuple=True)

    cell = tf.contrib.rnn.MultiRNNCell([lstm_cell() for _ in range(NUM_LAYERS)])

    output, _ = tf.nn.dynamic_rnn(cell, X, dtype=tf.float32)
    output = tf.reshape(output, [-1, HIDDEN_SIZE])

    # use fully connected layer to calculate predictions
    predictions = tf.contrib.layers.fully_connected(output, 1, None)
    predictions = tf.reshape(predictions, [-1])
    labels = tf.reshape(y, [-1])

    # define loss
    loss = tf.losses.mean_squared_error(predictions, labels)

    train_op = tf.contrib.layers.optimize_loss(loss, tf.contrib.framework.get_global_step(),
                                               optimizer="Adam", learning_rate=0.01)

    return predictions, loss, train_op


def create_regressor():
    regressor = SKCompat(learn.Estimator(model_fn=lstm_model, model_dir="Models/LSTMs1"))
    return regressor


def main():
    regressor = create_regressor()
    #
    # logdir = r"f:\log"
    # writer = tf.train.SummaryWriter(logdir, tf.get_dafault_graph())
    # writer.close()
    # print("tensorboard --logdir='%s'" % logdir)

    # fit model
    regressor.fit(train_X, train_y, batch_size=BATCH_SIZE,
                  steps=TRAINING_STEPS)

    # prediction
    predicted = [[pred] for pred in regressor.predict(test_X)]

    # calculate rmse
    rmse = np.sqrt(((predicted - test_y) ** 2).mean(axis=0))
    print("Root Mean Square Error is: %f" % rmse[0])


if __name__ == "__main__":
    main()




# -*- coding: utf-8 -*-

from sklearn import model_selection
from sklearn import metrics
import tensorflow as tf
import numpy as np
from tensorflow.contrib.learn.python.learn.estimators.estimator import SKCompat
learn = tf.contrib.learn


# 加载测试数据
def load_datas():

    from sklearn import datasets
    iris = datasets.load_iris()
    x_train, x_test, y_train, y_test = model_selection.train_test_split(
        iris.data, iris.target, test_size=0.2, random_state=0)

    x_train, x_test = map(np.float32, [x_train, x_test])

    return x_train, x_test, y_train, y_test


def softmax_in_tf(features, target, classes=3):
    target = tf.one_hot(target, classes, 1, 0)

    # 计算预测值及损失函数。
    logits = tf.contrib.layers.fully_connected(features, 3, tf.nn.softmax)
    loss = tf.losses.softmax_cross_entropy(target, logits)

    # 创建优化步骤。
    train_op = tf.contrib.layers.optimize_loss(
        loss,
        tf.contrib.framework.get_global_step(),
        optimizer='Adam',
        learning_rate=0.01)
    return tf.arg_max(logits, 1), loss, train_op


def main(x_train, y_train, x_test, y_test, model_dir):

    classifier = SKCompat(learn.Estimator(model_fn=softmax_in_tf, model_dir=model_dir))
    classifier.fit(x_train, y_train, steps=800)

    y_predicted = [i for i in classifier.predict(x_test)]
    score = metrics.accuracy_score(y_test, y_predicted)
    print('Accuracy: %.2f%%' % (score * 100))


if __name__ == "__main__":
    x_train, x_test, y_train, y_test = load_datas()
    main(x_train, y_train, x_test, y_test, model_dir='Models/softmax')



# -*- coding: utf-8 -*-

from wxpy import *

# 扫码登陆微信网页版
bot = Bot()  # 退出登录 bot.logout()

# 好友列表
friends = [str(f).strip('<Friend: >') for f in list(bot.friends())]

# 获取一个friend对象
my_friend = bot.friends().search('宋庆星')[0]


# 群列表
groups = [str(g).strip('<Group: >') for g in list(bot.groups())]

# 获取一个group对象
ng_group = bot.groups().search('Andrew ng Deep learning')[0]


sent_msgs = bot.messages


print(sent_msgs)




# -*- coding: utf-8 -*-

from __future__ import print_function
import pandas as pd
import numpy as np
from scipy import stats
import matplotlib.pyplot as plt
import statsmodels.api as sm
from statsmodels.graphics.api import qqplot
from zb_CodeSet.DataSet.load_dataset import load_monthly_milk_production


# step1. 输入数据，查看数据
dta = load_monthly_milk_production()
dta.plot(figsize=(12, 8))

# step2. 视情况，进行差分运算(通常从1阶差分开始)
fig = plt.figure(figsize=(12, 8))
ax1 = fig.add_subplot(111)
diff1 = dta['MilkProduction'].diff(1)  # 差分运算
diff1.plot(ax=ax1)


# step3. 确定合适的p, q参数
diff = dta.diff(1)  # 我们已经知道要使用一阶差分的时间序列，之前判断差分的程序可以注释掉
fig = plt.figure(figsize=(12, 8))
ax1 = fig.add_subplot(211)
fig = sm.graphics.tsa.plot_acf(diff, lags=40, ax=ax1)
ax2 = fig.add_subplot(212)
fig = sm.graphics.tsa.plot_pacf(diff, lags=40, ax=ax2)

# 根据 AIC / BIC / HQIC判断
arma_mod20 = sm.tsa.ARMA(dta, (7, 0)).fit()
print(arma_mod20.aic, arma_mod20.bic, arma_mod20.hqic)
arma_mod30 = sm.tsa.ARMA(dta, (0, 1)).fit()
print(arma_mod30.aic, arma_mod30.bic, arma_mod30.hqic)
arma_mod40 = sm.tsa.ARMA(dta, (7, 1)).fit()
print(arma_mod40.aic, arma_mod40.bic, arma_mod40.hqic)
arma_mod50 = sm.tsa.ARMA(dta, (8, 0)).fit()
print(arma_mod50.aic, arma_mod50.bic, arma_mod50.hqic)






# -*- coding: utf-8 -*-
"""
——————————————————————————————————————————————————————————————————————————————————————————
Send Message Service
——————————————————————————————————————————————————————————————————————————————————————————
"""


from retrying import retry
import os


"""
——————————————————————————————————————————————————————————————————————————————————————————
使用server酱推送消息
——————————————————————————————————————————————————————————————————————————————————————————
"""


import requests

@retry(stop_max_attempt_number=6)
def push_sms(title, content, key):
    """使用server酱推送消息到微信"""
    # post方法推送
    url = 'https://sc.ftqq.com/%s.send' % key
    requests.post(url, data={'text': title, 'desp': content})


"""
——————————————————————————————————————————————————————————————————————————————————————————
邮件发送模块，支持发送附件。
——————————————————————————————————————————————————————————————————————————————————————————
"""

import smtplib
from email.mime.application import MIMEApplication
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText


class EmailSender:
    """
    example
    ----------------
    subject = '附件发送测试'
    content = '检测附件发送是否成功'
    files = ['test.py',
             'f:\\已下载.txt']

    e = EmailSender('zeng_bin8888@163.com', pw=pw, service='163')
    e.send_email(to, subject, content, files)
    """
    def __init__(self, from_, pw, service='qq'):
        self.from_ = from_  # 用于发送email的邮箱
        self.pw = pw        # 发送email的邮箱密码
        self.smtp = smtplib.SMTP()
        if service == 'qq':
            self.smtp.connect('smtp.exmail.qq.com')
            self.smtp.login(self.from_, self.pw)
        elif service == '163':
            self.smtp.connect('smtp.163.com')
            self.smtp.login(self.from_, self.pw)
        else:
            print('目前仅支持163和qq邮箱！')


    def _construct_msg(self, to, subject, content, files=None):
        """构造email信息

        parameters
        ---------------
        subject     邮件主题
        content     邮件文本内容
        files       附件（list）
                    可以是相对路径下的文件，也可以是绝对路径下的文件；
                    推荐使用绝对路径。

        return
        --------------
        msg         构造好的邮件信息
        """
        msg = MIMEMultipart()
        msg['from'] = self.from_
        msg['to'] = to
        msg['subject'] = subject
        txt = MIMEText(content)
        msg.attach(txt)

        # 添加附件
        if files is not None:
            for file in files:
                f = MIMEApplication(open(file, 'rb').read())
                f.add_header('Content-Disposition', 'attachment',
                             filename=os.path.split(file)[-1])
                msg.attach(f)

        return msg


    @retry(stop_max_attempt_number=6)
    def send_email(self, to, subject, content, files=None):
        """登录邮箱，发送msg到指定联系人

        :param to: str: 收件人邮箱
        :param subject: str: 主题
        :param content: str: 内容
        :param files:  list: 附件列表
        :return: None
        """
        smtp = self.smtp
        msg = EmailSender._construct_msg(self, to, subject, content, files=files)
        smtp.sendmail(self.from_, to, str(msg))


    def quit(self):
        smtp = self.smtp
        smtp.quit()  # 退出登录



# -*- coding: utf-8 -*-
"""
----------------------------------------------------------------------------------
Database Connector
1. use sqlalchemy.create_engine to create database engine
2. connect to database
3. execute SQL
4. close connection
----------------------------------------------------------------------------------
"""

import os
import subprocess
import pandas as pd
from sqlalchemy import create_engine


"""
----------------------------------------------------------------------------------
Mongodb Connector
----------------------------------------------------------------------------------
"""
class MongoConnector:
    """MongoDB"""
    from pymongo import MongoClient
    client = MongoClient('localhost', 27107)
    # client = MongoClient('mongodb://localhost:27017/')
    db = client['tes_database']
    collection = db['test_collection']

"""
----------------------------------------------------------------------------------
SQLite3 Connector
----------------------------------------------------------------------------------
"""
import sqlite3


class SQLiteConnector:
    """SQLite数据库连接器"""
    def __init__(self, db):
        self.db = db
        self.conn = sqlite3.connect(db)
        self.cursor = self.conn.cursor()

    def execute_sql_with_commit(self, sql):
        """执行sql，并commit"""
        self.cursor.execute(sql)
        self.conn.commit()

    def select_to_df(self, sql):
        """执行sql查询语句，返回DataFrame"""
        datas = pd.read_sql(sql, con=self.conn)
        return datas

    def df_to_db(self, df, table_name, if_exists='fail'):
        """将DataFrame数据保存到数据库"""
        df.to_sql(table_name, con=self.conn, index=False, if_exists=if_exists)

    def vacuum(self):
        """删除数据后释放空间"""
        self.cursor.execute('VACUUM')
        self.conn.commit()

    def close(self):
        self.conn.close()


"""
----------------------------------------------------------------------------------
Oracle Connector
cx_Oracle is needed, if do not have one, execute 'pip install cx_Oracle'.
----------------------------------------------------------------------------------
"""
try:
    import cx_Oracle
except ImportError:
    os.system('pip install cx_Oracle')
    import cx_Oracle

class OracleConnector:

    def __init__(self, user, pw, ip, service):
        # 连接oracle数据库
        self.conn = cx_Oracle.connect(user, pw, ip + '/' + service)
        # conn = cx_Oracle.connect('c##bzeng/c##bzeng@192.168.31.103/orcl103')
        # 获取cursor
        self.cursor = self.conn.cursor()
        # 使用sqlalchemy创建数据库引擎
        self.ora = create_engine("oracle://%s:%s@%s:1521/%s" % (user, pw, ip, service))


    def read_sql(self, sql):
        """读取查询语句的结果"""
        df = pd.read_sql(sql, con=self.ora)
        return df


    def to_sql(self, df, table=None):
        assert table is not None, "table变量不能为空，请填入表名"
        df.to_sql(name=table, con=self.ora, if_exists='append')


    def close(self):
        self.conn.close()



"""
----------------------------------------------------------------------------------
Mysql Connector
----------------------------------------------------------------------------------
"""


class MysqlConnector:
    def __init__(self):
        try:
            import pymysql
        except ImportError:
            subprocess.run(['pip', 'install', 'pymysql'])
            import pymysql

    # # 创建连接
    # conn = pymysql.connect(host='127.0.0.1', port=3306, user='root', passwd='', db='tkq1', charset='utf8')
    # # 创建游标
    # cursor = conn.cursor()
    # # 执行SQL，并返回受影响行数
    # effect_row = cursor.execute("select * from tb7")

def mysql_test():
    import pymysql

    host = 'localhost'
    username = 'root'
    password = '1111'
    db_name = 'test1'

    connection = pymysql.connect(host=host,
                                 user=username,
                                 password=password,
                                 charset='utf8',
                                 db=db_name)
    with connection as conn:
        cursor = conn.cursor()
        cursor.execute('select * from table')
        results = cursor.fetchall()
        for row in results:
            print(row[0], row[1], row[2], row[3], sep='\t')
        conn.commit()
        conn.close()




# -*- coding: utf-8 -*-
"""
Create Date: 2017-07-24
python代码分析工具
输入：
    单个（多个）脚本路径  or   项目路径

输出：
    分析结果（md文件）

功能：
    1、统计单个脚本中定义的函数、类的数量，并将函数名和类名分别保存
    2、

"""

import os

py_path = r'C:\Anaconda3\Lib\zb_CodeSet\local_use'
fn_pys = [os.path.join(py_path, fn) for fn in os.listdir(py_path)]

def read_py(fn_py):
    """将py文件读入"""
    with open(fn_py, 'r', encoding='utf-8') as pyf:
        lines = pyf.readlines()
    return lines

def save_py(save_lines, fn_py):
    """保存处理之后的代码"""
    with open(fn_py, 'w', encoding='utf-8') as pyf:
        pyf.writelines(save_lines)

def del_annotation(py_lines):
    """删除以#开头的单行注释"""
    save_lines = []
    for i in range(1, len(py_lines)):
        if not py_lines[i].strip(' ').startswith('#'):
            save_lines.append(py_lines[i])
    return save_lines

# fn_py = fn_pys[1]
# py_lines = read_py(fn_py)
# save_lines = del_annotation(py_lines)
# save_py(save_lines, fn_py)



# -*- coding: utf-8 -*-
"""
Create date: 2017-07-17
@Author : zengbin
A tool for init ubuntu.
"""

import os
import subprocess

wdir = '/home/bzeng/Downloads'

def prepare_env():
    install = 'sudo apt-get install '
    os.system(install + 'vim')
    os.system(install + 'openssh-server')

def batch_download(path=wdir):
    """batch download files needed"""
    os.chdir(path)
    download_lists = {
                'Hadoop 2.7.3': 'https://mirrors.tuna.tsinghua.edu.cn/apache/hadoop/common/hadoop-2.7.3/hadoop-2.7.3.tar.gz',
                'Hive 2.1.1': 'https://mirrors.tuna.tsinghua.edu.cn/apache/hive/hive-2.1.1/apache-hive-2.1.1-bin.tar.gz',
                'Spark 2.2.0': 'https://mirrors.tuna.tsinghua.edu.cn/apache/spark/spark-2.2.0/spark-2.2.0-bin-hadoop2.7.tgz',
                'Hbase 1.2.6': 'https://mirrors.tuna.tsinghua.edu.cn/apache/hbase/stable/hbase-1.2.6-bin.tar.gz',
                'Pig 0.17': 'https://mirrors.tuna.tsinghua.edu.cn/apache/pig/latest/pig-0.17.0.tar.gz',
                'jdk 8u131': 'http://download.oracle.com/otn-pub/java/jdk/8u131-b11/d54c1d3a095b4ff2b6607d096fa80163/jdk-8u131-linux-x64.tar.gz',
    }

    for key, url in download_lists.items():
        print('current downloading: %s' % key)
        os.system('nohup wget ' + url + '&')

    os.chdir(wdir)


def batch_decompress():
    """"""
    pass

"""
-------------------------------------------------------------------------------
Install Docker on Ubuntu 16.04
-------------------------------------------------------------------------------
"""
cmd1 = 'apt-get update'
cmd2 = 'apt-get install apt-transport-https ca-certificates'
cmd3 = 'apt-key adv --keyserver hkp://p80.pool.sks-keyservers.net:80 --recv-keys 58118E89F3A912897C070ADBF76221572C52609D'
cmd4 = ''
p = subprocess.run(cmd1.split(' '))



"""
-------------------------------------------------------------------------------
Install MongoDB on Ubuntu 16.04
https://docs.mongodb.com/manual/tutorial/install-mongodb-on-ubuntu/
-------------------------------------------------------------------------------
"""
cmd = 'su'
# step 1. Import the public key used by the package management system.
mongodb_cmd1 = 'sudo apt-key adv --keyserver hkp://keyserver.ubuntu.com:80 --recv 0C49F3730359A14518585931BC711F9BA15703C6'
# step 2. Create a list file for MongoDB.
mongodb_cmd2 = 'echo "deb [ arch=amd64,arm64 ] http://repo.mongodb.org/apt/ubuntu xenial/mongodb-org/3.4 multiverse" | sudo tee /etc/apt/sources.list.d/mongodb-org-3.4.list'
# step 3. Reload local package database.
mongodb_cmd3 = 'sudo apt-get update'
# step 4. Install the MongoDB packages.
mongodb_install = 'sudo apt-get install -y mongodb-org'



from setuptools import setup, find_packages
import zb

setup(
    name="zb",
    version=zb.__version__,
    keywords=("zb", "tool", "algorithm"),
    description="My Personal Package",
    long_description="Personal established tools and implement of algorithm. ",
    license="Apache-2.0",

    url="https://github.com/zengbin93/zb",
    author=zb.__author__,
    author_email="zeng_bin8888@163.com",

    packages=find_packages(exclude=['code_set', 'test', 'images']),
    include_package_data=True,
    install_requires=[
        "requests", "pandas", "click", 'retrying',
        "numpy",
    ],
    entry_points={
        "console_scripts": [
            "zb=zb.cli:zb"
        ]
    }
)



# -*- coding:utf-8 -*-
import os
import time
import requests
from retrying import retry
import click

@retry(stop_max_attempt_number=6)
def push_sms(title, content, key='SCU10748T12f471f07094648d297222fc649e374d598bf38bc81fd'):
    """使用server酱推送消息到微信"""
    # post方法推送
    url = 'https://sc.ftqq.com/%s.send' % key
    requests.post(url, data={'text': title, 'desp': content})


class Guard:
    alive = "I am still running"
    dead = "I got an Exception, please save me"
    stop = "I stopped myself, leave me alone"
    file = "state.txt"

    def write_state(self, state):
        with open(self.file, 'w', encoding='utf8') as f:
            if state == "alive":
                f.write(self.alive)
            elif state == "dead":
                f.write(self.dead)
            elif state == "stop":
                f.write(self.stop)

    def response_state(self, response_cmd):
        if not os.path.exists(self.file):
            print("执行cmd：%s" % response_cmd)
            os.system(response_cmd)
            time.sleep(100)
            return "first run"
        with open(self.file, 'r', encoding='utf8') as f:
            state = f.readline()
            if state == self.dead:
                push_sms(title="程序死掉了", content=state)
                os.system(response_cmd)
                push_sms(title="程序复活", content="执行命令：%s" % response_cmd)
            # elif state == self.alive:
            #     print(state)
            elif state == self.stop:
                push_sms(title="程序关机提醒", content=state)
        return state



@click.command()
@click.option('--cmd', help="程序死掉之后的响应命令")
@click.option('--sleep', default=10, help="休眠时间")
def guard(cmd, sleep):
    guard = Guard()
    while True:
        state = guard.response_state(response_cmd=cmd)
        if state == guard.stop:
            break
        time.sleep(sleep)


if __name__ == "__main__":
    guard()



# -*- coding: utf-8 -*-

import functools
import time


def time_cost(func):
    """装饰器：计算函数运行耗时"""
    @functools.wraps(func)
    def wrapper(*args, **kw):
        start = time.time()
        print('正在运行函数 %s() ...' % func.__name__)
        res = func(*args, **kw)
        end = time.time()
        print('函数【 %s() 】运行耗时：%.2f 秒' % (func.__name__, end - start))
        return res
    return wrapper
